/*
 * Copyright (c) 2017-2022, Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 */

#include "os_wrapper/thread.h"
#include "os_wrapper/mutex.h"
#include "os_wrapper/semaphore.h"
#include "os_wrapper/delay.h"
#if (defined(TFM_NS_RTOS2_WRAPPER_RTOS2_RTX) || \
     defined(TFM_NS_RTOS2_WRAPPER_RTOS2_FREERTOS))
#include "cmsis_os2.h"
#endif
#include "timer_interface.h"

#include "tfm_nsid_manager.h"
#ifdef TFM_NS_MANAGE_NSID
#include "tfm_nsid_map_table.h"
#endif

#ifndef UNUSED_VARIABLE
#define UNUSED_VARIABLE(X) ((void)(X))
#endif

#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)//YX 20221219
static uint32_t TempSemaphoreId = 0;
static uint32_t TempThreadId = 0;
static uint32_t TempMutexId = 0;
static uint32_t TempMessageQueueId = 0;
#endif

/* This is an example OS abstraction layer for CMSIS-RTOSv2 */

void *os_wrapper_thread_new(const char *name, int32_t stack_size,
                            os_wrapper_thread_func func, void *arg,
                            uint32_t priority)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(name);
    UNUSED_VARIABLE(stack_size);
    UNUSED_VARIABLE(priority);
    func(arg);
    return (void *)&TempThreadId;
#else

    osThreadAttr_t task_attribs = {
        .tz_module = ((TZ_ModuleId_t)TFM_DEFAULT_NSID)
    };

    /* By default, the thread starts as osThreadDetached */
    if (stack_size != OS_WRAPPER_DEFAULT_STACK_SIZE) {
        task_attribs.stack_size = stack_size;
    }
    task_attribs.name = name;
    task_attribs.priority = (osPriority_t) priority;

#ifdef TFM_NS_MANAGE_NSID
    task_attribs.tz_module = (TZ_ModuleId_t)nsid_mgr_get_thread_nsid(name);
#endif

    return (void *)osThreadNew(func, arg, &task_attribs);
#endif
}

void *os_wrapper_semaphore_create(uint32_t max_count, uint32_t initial_count,
                                  const char *name)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(max_count);
    UNUSED_VARIABLE(initial_count);
    UNUSED_VARIABLE(name);
    return &TempSemaphoreId;
#else
    osSemaphoreAttr_t sema_attrib = {0};

    sema_attrib.name = name;

    return (void *)osSemaphoreNew(max_count, initial_count, &sema_attrib);
#endif
}

uint32_t os_wrapper_semaphore_acquire(void *handle, uint32_t timeout)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    UNUSED_VARIABLE(timeout);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status;

    status = osSemaphoreAcquire((osSemaphoreId_t)handle,
                                (timeout == OS_WRAPPER_WAIT_FOREVER) ?
                                osWaitForever : timeout);
    if (status != osOK) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

uint32_t os_wrapper_semaphore_release(void *handle)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status;

    status = osSemaphoreRelease((osSemaphoreId_t)handle);
    if (status != osOK) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

uint32_t os_wrapper_semaphore_delete(void *handle)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status;

    status = osSemaphoreDelete((osSemaphoreId_t)handle);
    if (status != osOK) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

void *os_wrapper_mutex_create(void)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    return &TempMutexId;
#else
    const osMutexAttr_t attr = {
        .name = NULL,
        .attr_bits = osMutexPrioInherit, /* Priority inheritance is recommended
                                          * to enable if it is supported.
                                          * For recursive mutex and the ability
                                          * of auto release when owner being
                                          * terminated is not required.
                                          */
        .cb_mem = NULL,
        .cb_size = 0U
    };

    return (void *)osMutexNew(&attr);
#endif
}

uint32_t os_wrapper_mutex_acquire(void *handle, uint32_t timeout)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    UNUSED_VARIABLE(timeout);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status = osOK;

    if (!handle) {
        return OS_WRAPPER_ERROR;
    }

    status = osMutexAcquire((osMutexId_t)handle,
                            (timeout == OS_WRAPPER_WAIT_FOREVER) ?
                             osWaitForever : timeout);
    if (status != osOK) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

uint32_t os_wrapper_mutex_release(void *handle)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status = osOK;

    if (!handle) {
        return OS_WRAPPER_ERROR;
    }

    status = osMutexRelease((osMutexId_t)handle);
    if (status != osOK) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

uint32_t os_wrapper_mutex_delete(void *handle)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status = osOK;

    if (!handle) {
        return OS_WRAPPER_ERROR;
    }

    status = osMutexDelete((osMutexId_t)handle);
    if (status != osOK) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

void *os_wrapper_thread_get_handle(void)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    return &TempThreadId;
#else
    return (void *)osThreadGetId();
#endif
}

uint32_t os_wrapper_thread_get_priority(void *handle, uint32_t *priority)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    *priority = 0;//osPriorityLow;
    return OS_WRAPPER_SUCCESS;
#else
    osPriority_t prio;

    prio = osThreadGetPriority((osThreadId_t)handle);
    if (prio == osPriorityError) {
        return OS_WRAPPER_ERROR;
    }

    *priority = (uint32_t)prio;

    return OS_WRAPPER_SUCCESS;
#endif
}

void os_wrapper_thread_exit(void)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)

#else
    osThreadExit();
#endif
}

uint32_t os_wrapper_thread_set_flag(void *handle, uint32_t flags)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    UNUSED_VARIABLE(flags);
    return OS_WRAPPER_SUCCESS;
#else
    uint32_t ret;

    ret = osThreadFlagsSet((osThreadId_t)handle, flags);
    if (ret & osFlagsError) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

/*
 * According to the description of CMSIS-RTOS v2 Thread Flags,
 * osThreadFlagsSet() can be called inside Interrupt Service Routine.
 */
uint32_t os_wrapper_thread_set_flag_isr(void *handle, uint32_t flags)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(handle);
    UNUSED_VARIABLE(flags);
    return OS_WRAPPER_SUCCESS;
#else
    uint32_t ret;

    ret = osThreadFlagsSet((osThreadId_t)handle, flags);
    if (ret & osFlagsError) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

uint32_t os_wrapper_thread_wait_flag(uint32_t flags, uint32_t timeout)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(flags);
    UNUSED_VARIABLE(timeout);
    return OS_WRAPPER_SUCCESS;
#else
    uint32_t ret;

    ret = osThreadFlagsWait(flags, osFlagsWaitAll,
                            (timeout == OS_WRAPPER_WAIT_FOREVER) ?
                            osWaitForever : timeout);
    if (ret & osFlagsError) {
        return OS_WRAPPER_ERROR;
    }

    return OS_WRAPPER_SUCCESS;
#endif
}

uint32_t os_wrapper_get_tick(void)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    return 0;
#else
    return osKernelGetTickCount();
#endif
}

void *os_wrapper_msg_queue_create(size_t msg_size, uint8_t msg_count)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(msg_size);
    UNUSED_VARIABLE(msg_count);
    return &TempMessageQueueId;
#else
    osMessageQueueId_t mq_id;

    mq_id = osMessageQueueNew(msg_count, msg_size, NULL);

    return (void *)mq_id;
#endif
}

int32_t os_wrapper_msg_queue_send(void *mq_handle,
                                  const void *msg_ptr)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(mq_handle);
    UNUSED_VARIABLE(msg_ptr);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status;

    status = osMessageQueuePut(mq_handle, msg_ptr, 0, 0);
    if (status == osOK) {
        return OS_WRAPPER_SUCCESS;
    }

    return OS_WRAPPER_ERROR;
#endif
}

int32_t os_wrapper_msg_queue_receive(void *mq_handle,
                                     void *msg_ptr)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    UNUSED_VARIABLE(mq_handle);
    UNUSED_VARIABLE(msg_ptr);
    return OS_WRAPPER_SUCCESS;
#else
    osStatus_t status;

    status = osMessageQueueGet(mq_handle, msg_ptr, NULL, osWaitForever);
    if (status == osOK) {
        return OS_WRAPPER_SUCCESS;
    }

    return OS_WRAPPER_ERROR;
#endif
}

int32_t os_wrapper_delay(uint32_t ticks)
{
#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
#if 0
    volatile uint32_t i = 0;
    volatile uint32_t count;

    for (i = 0; i < ticks; i++) {
        count = 10000;
        while (count--)
        {
        }
    }

    return OS_WRAPPER_SUCCESS;
#else
    if (ticks != 0U) {
        hx_drv_timer_cm55x_delay_ms(ticks, TIMER_STATE_DC);
    }

    return OS_WRAPPER_SUCCESS;
#endif
#else
    osStatus_t status;

    status = osDelay(ticks);
    if (status == osOK) {
        return OS_WRAPPER_SUCCESS;
    }

    return OS_WRAPPER_ERROR;
#endif
}
