/*
 * Copyright (c) 2013-2022 Arm Limited. All rights reserved.
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Licensed under the Apache License, Version 2.0 (the License); you may
 * not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <string.h>
#include <stdint.h>
#include "Driver_Flash.h"
#include "cmsis_driver_config.h"
//#include "RTE_Device.h"
#include "platform_base_address.h"
#include "emulated_flash_drv.h"
#include "flash_layout.h" 

#ifndef ARG_UNUSED
#define ARG_UNUSED(arg)  ((void)arg)
#endif

//#define dbg_log(fmt, ...)  printf(fmt, ##__VA_ARGS__)
#define dbg_log(fmt, ...)

/* Driver version */
#define ARM_FLASH_DRV_VERSION      ARM_DRIVER_VERSION_MAJOR_MINOR(1, 1)

/**
 * Data width values for ARM_FLASH_CAPABILITIES::data_width
 * \ref ARM_FLASH_CAPABILITIES
 */
 enum {
    DATA_WIDTH_8BIT   = 0u,
    DATA_WIDTH_16BIT,
    DATA_WIDTH_32BIT,
    DATA_WIDTH_ENUM_SIZE
};

static const uint32_t data_width_byte[DATA_WIDTH_ENUM_SIZE] = {
    sizeof(uint8_t),
    sizeof(uint16_t),
    sizeof(uint32_t),
};

/* Driver Version */
static const ARM_DRIVER_VERSION DriverVersion = {
    ARM_FLASH_API_VERSION,
    ARM_FLASH_DRV_VERSION
};

/* Driver Capabilities */
static const ARM_FLASH_CAPABILITIES DriverCapabilities = {
    0, /* event_ready */
    0, /* data_width = 0:8-bit, 1:16-bit, 2:32-bit */
    0, /* erase_chip */
    0  /* reserved */
};

static ARM_DRIVER_VERSION ARM_Flash_GetVersion(void)
{
    return DriverVersion;
}

static ARM_FLASH_CAPABILITIES ARM_Flash_GetCapabilities(void)
{
    return DriverCapabilities;
}

#if !defined(IMPLEMENT_FLASH_DRV)
static int32_t ARM_Flash_Initialize(ARM_Flash_SignalEvent_t cb_event)
{
    ARG_UNUSED(cb_event);
    dbg_log("ARM_Flash_Initialize\r\n");
    if (DriverCapabilities.data_width >= DATA_WIDTH_ENUM_SIZE) {
        return ARM_DRIVER_ERROR;
    }

    /* Nothing to be done */
    return ARM_DRIVER_OK;
}
#else
static int32_t ARM_Flash_Initialize(ARM_Flash_SignalEvent_t cb_event)
{
    ARG_UNUSED(cb_event);
    static int init_done = 0;

    dbg_log("ARM_Flash_Initialize\r\n");

    if (init_done == 0) {
        //int rc = hx_lib_spi_eeprom_open_speed(USE_DW_SPI_MST_Q, 60000);
        int rc = hx_lib_spi_eeprom_open(USE_DW_SPI_MST_Q);
        //int rc = hx_lib_spi_eeprom_open_speed(USE_DW_SPI_MST_Q, 12000000);

        if (E_OK != rc)
            return ARM_DRIVER_ERROR;

        init_done = 1;
    }

    if (DriverCapabilities.data_width >= DATA_WIDTH_ENUM_SIZE) {
        return ARM_DRIVER_ERROR;
    }

    /* Nothing to be done */
    return ARM_DRIVER_OK;
}
#endif

static int32_t ARM_Flash_Uninitialize(void)
{
    /* Nothing to be done */
    dbg_log("ARM_Flash_Uninitialize\r\n");
    return ARM_DRIVER_OK;
}

static int32_t ARM_Flash_PowerControl(ARM_POWER_STATE state)
{
    dbg_log("ARM_Flash_PowerControl\r\n");
    switch (state) {
    case ARM_POWER_FULL:
        /* Nothing to be done */
        return ARM_DRIVER_OK;

    case ARM_POWER_OFF:
    case ARM_POWER_LOW:
    default:
        return ARM_DRIVER_ERROR_UNSUPPORTED;
    }
}

#if !defined(IMPLEMENT_FLASH_DRV)
#define FLASH_1_BASE_S         BASE_ADDR_SRAM2_ALIAS
#define FLASH_1_BASE_NS        BASE_ADDR_SRAM2
#define FLASH_1_SIZE           SRAM2_SIZE
#else
#define FLASH_1_SIZE           FLASH_TOTAL_SIZE
#ifndef HX_CMSIS_FLASH1_ADDR
#define HX_CMSIS_FLASH1_ADDR          0x800000  /* Flash base address */
#endif
#ifndef HX_CMSIS_FLASH1_SIZE
#define HX_CMSIS_FLASH1_SIZE         ((FLASH_TOTAL_SIZE - HX_CMSIS_FLASH1_ADDR) / 4)
#endif
#endif

#define FLASH_1_SECTOR_SIZE    0x00001000 /* 4 kB */
#define FLASH_1_PAGE_SIZE      256        /* 256 B */
#define FLASH_1_PROGRAM_UNIT   0x1        /* Minimum write size */


/* Flash Status */
static ARM_FLASH_STATUS FlashStatus_FLASH1 = {0, 0, 0};

static ARM_FLASH_INFO ARM_FLASH1_DEV_INFO = {
    .sector_info  = NULL,                  /* Uniform sector layout */
#if !defined(IMPLEMENT_FLASH_DRV)
    .sector_count = FLASH_1_SIZE / FLASH_1_SECTOR_SIZE,
#else
    .sector_count = HX_CMSIS_FLASH1_SIZE / FLASH_1_SECTOR_SIZE,
#endif
    .sector_size  = FLASH_1_SECTOR_SIZE,
    .page_size    = FLASH_1_PAGE_SIZE,
    .program_unit = FLASH_1_PROGRAM_UNIT,
    .erased_value = EMULATED_FLASH_DRV_ERASE_VALUE};

#if !defined(IMPLEMENT_FLASH_DRV)
static struct emulated_flash_dev_t ARM_FLASH1_DEV = {
#if (defined (__DOMAIN_NS) && (__DOMAIN_NS == 1))
    .memory_base_ns = FLASH_1_BASE_NS,
#else
    .memory_base_ns = FLASH_1_BASE_NS,
    .memory_base_s = FLASH_1_BASE_S,
#endif /* __DOMAIN_NS == 1 */
    .data        = &(ARM_FLASH1_DEV_INFO)};

struct emulated_flash_dev_t *FLASH1_DEV = &ARM_FLASH1_DEV;
#endif

/*
 * Functions
 */
#if !defined(IMPLEMENT_FLASH_DRV)
static int32_t ARM_Flash_ReadData(uint32_t addr, void *data, uint32_t cnt)
{
    /* Conversion between data items and bytes */
    cnt *= data_width_byte[DriverCapabilities.data_width];
    dbg_log("ARM_Flash_ReadData addr 0x%X cnt 0x%X\r\n", addr, cnt);

    enum emulated_flash_error_t rc = emulated_flash_read_data(FLASH1_DEV, addr, data, cnt);
    if (EMULATED_FLASH_ERR_NONE == rc) {
        cnt /= data_width_byte[DriverCapabilities.data_width];
        return cnt;
    } else if(EMULATED_FLASH_ERR_INVALID_PARAM == rc) {
        return ARM_DRIVER_ERROR_PARAMETER;
    } else {
        return ARM_DRIVER_ERROR;
    }
}
#else
static int32_t ARM_Flash_ReadData(uint32_t addr, void *data, uint32_t cnt)
{
    /* Conversion between data items and bytes */
    cnt *= data_width_byte[DriverCapabilities.data_width];
    addr += HX_CMSIS_FLASH1_ADDR;
    dbg_log("ARM_Flash_ReadData addr 0x%X cnt 0x%X\r\n", addr, cnt);

    int rc = hx_lib_spi_eeprom_2read(USE_DW_SPI_MST_Q, addr, (uint8_t *)data, cnt);

    if (E_OK == rc) {
        cnt /= data_width_byte[DriverCapabilities.data_width];
        return cnt;
    //} else if(EMULATED_FLASH_ERR_INVALID_PARAM == rc) {
    //    return ARM_DRIVER_ERROR_PARAMETER;
    } else {
        return ARM_DRIVER_ERROR;
    }
}
#endif

#if !defined(IMPLEMENT_FLASH_DRV)
static int32_t ARM_Flash_ProgramData(uint32_t addr, const void *data,
                                     uint32_t cnt)
{
    /* Conversion between data items and bytes */
    cnt *= data_width_byte[DriverCapabilities.data_width];
    dbg_log("ARM_Flash_ProgramData addr 0x%X cnt 0x%X\r\n", addr, cnt);
    enum emulated_flash_error_t rc = emulated_flash_program_data(FLASH1_DEV, addr, data, cnt);

    if (EMULATED_FLASH_ERR_NONE == rc) {
        cnt /= data_width_byte[DriverCapabilities.data_width];
        return (int32_t)cnt;
    } else if(EMULATED_FLASH_ERR_INVALID_PARAM == rc) {
        return ARM_DRIVER_ERROR_PARAMETER;
    } else if(EMULATED_FLASH_NOT_READY == rc) {
        return ARM_DRIVER_ERROR;
    } else {
        return ARM_DRIVER_ERROR;
    }
}
#else

static int32_t ARM_Flash_ProgramData(uint32_t addr, const void *data,
                                     uint32_t cnt)
{
    /* Conversion between data items and bytes */
    cnt *= data_width_byte[DriverCapabilities.data_width];
    addr += HX_CMSIS_FLASH1_ADDR;
    dbg_log("ARM_Flash_ProgramData addr 0x%X cnt 0x%X\r\n", addr, cnt);

    int rc = hx_lib_spi_eeprom_write(USE_DW_SPI_MST_Q, addr, (uint8_t *)data, cnt, 0);

    if (E_OK == rc) {
        cnt /= data_width_byte[DriverCapabilities.data_width];
        return (int32_t)cnt;
    //} else if(EMULATED_FLASH_ERR_INVALID_PARAM == rc) {
    //    return ARM_DRIVER_ERROR_PARAMETER;
    //} else if(EMULATED_FLASH_NOT_READY == rc) {
    //    return ARM_DRIVER_ERROR;
    } else {
        return ARM_DRIVER_ERROR;
    }
}
#endif
#if !defined(IMPLEMENT_FLASH_DRV)
static int32_t ARM_Flash_EraseSector(uint32_t addr)
{
    dbg_log("ARM_Flash_EraseSector addr 0x%X\r\n", addr);
    enum emulated_flash_error_t rc = emulated_flash_erase_sector(FLASH1_DEV, addr);

    if (EMULATED_FLASH_ERR_NONE == rc) {
        return ARM_DRIVER_OK;
    } else if(EMULATED_FLASH_ERR_INVALID_PARAM == rc) {
        return ARM_DRIVER_ERROR_PARAMETER;
    } else {
        return ARM_DRIVER_ERROR;
    }
}
#else
static int32_t ARM_Flash_EraseSector(uint32_t addr)
{
    dbg_log("ARM_Flash_EraseSector addr 0x%X\r\n", addr);
    addr += HX_CMSIS_FLASH1_ADDR;
    int rc = hx_lib_spi_eeprom_erase_sector(USE_DW_SPI_MST_Q, addr, FLASH_SECTOR);
    //int rc = hx_lib_spi_eeprom_erase_sector(USE_DW_SPI_MST_Q, addr);

    if (E_OK == rc) {
        return ARM_DRIVER_OK;
    //} else if(EMULATED_FLASH_ERR_INVALID_PARAM == rc) {
    //    return ARM_DRIVER_ERROR_PARAMETER;
    } else {
        return ARM_DRIVER_ERROR;
    }
}
#endif
#if !defined(IMPLEMENT_FLASH_DRV)
static int32_t ARM_Flash_EraseChip(void)
{
    dbg_log("ARM_Flash_EraseChip\r\n");
    //emulated_flash_erase_chip(FLASH1_DEV);
    return ARM_DRIVER_ERROR_UNSUPPORTED;
}
#else
static int32_t ARM_Flash_EraseChip(void)
{
    dbg_log("ARM_Flash_EraseChip\r\n");
    //hx_lib_spi_eeprom_erase_all(USE_DW_SPI_MST_Q);
    return ARM_DRIVER_ERROR_UNSUPPORTED;
}
#endif

static ARM_FLASH_INFO * ARM_Flash_GetInfo(void)
{
    return &ARM_FLASH1_DEV_INFO;
}

static ARM_FLASH_STATUS ARM_Flash_GetStatus(void)
{
    return FlashStatus_FLASH1;
}

ARM_DRIVER_FLASH Driver_Flash1 = {
    ARM_Flash_GetVersion,
    ARM_Flash_GetCapabilities,
    ARM_Flash_Initialize,
    ARM_Flash_Uninitialize,
    ARM_Flash_PowerControl,
    ARM_Flash_ReadData,
    ARM_Flash_ProgramData,
    ARM_Flash_EraseSector,
    ARM_Flash_EraseChip,
    ARM_Flash_GetStatus,
    ARM_Flash_GetInfo
};
