/*
 * Copyright (c) 2019-2022 Arm Limited. All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "cmsis.h"
#include "utilities.h"
#include "target_cfg.h"
//#include "Driver_SSE300_PPC.h"
//#include "Driver_MPC.h"
#include "region_defs.h"
#include "tfm_plat_defs.h"
#include "region.h"
#include "device_definition.h"
//#include "syscounter_armv8-m_cntrl_drv.h"
#include "uart_stdout.h"

/* Throw out bus error when an access causes security violation */
#define CMSDK_SECRESPCFG_BUS_ERR_MASK   (1UL << 0)

/* The section names come from the scatter file */
REGION_DECLARE(Load$$LR$$, LR_NS_PARTITION, $$Base);
REGION_DECLARE(Image$$, ER_VENEER, $$Base);
REGION_DECLARE(Image$$, VENEER_ALIGN, $$Limit);

const struct memory_region_limits memory_regions = {
    .non_secure_code_start =
//        (uint32_t)&REGION_NAME(Load$$LR$$, LR_NS_PARTITION, $$Base) +
//        BL2_HEADER_SIZE,
        (uint32_t)&REGION_NAME(Load$$LR$$, LR_NS_PARTITION, $$Base),

    .non_secure_partition_base =
        (uint32_t)&REGION_NAME(Load$$LR$$, LR_NS_PARTITION, $$Base),

    .non_secure_partition_limit =
        (uint32_t)&REGION_NAME(Load$$LR$$, LR_NS_PARTITION, $$Base) +
        NS_PARTITION_SIZE - 1,

    .veneer_base = (uint32_t)&REGION_NAME(Image$$, ER_VENEER, $$Base),
    .veneer_limit = (uint32_t)&REGION_NAME(Image$$, VENEER_ALIGN, $$Limit),
};

/* Configures the RAM region to NS callable in sacfg block's nsccfg register */
#define RAMNSC  0x2
/* Configures the CODE region to NS callable in sacfg block's nsccfg register */
#define CODENSC  0x1

/* Define Peripherals NS address range for the platform */
#define PERIPHERALS_BASE_NS_START      (0x40000000)
//#define PERIPHERALS_BASE_NS_END        (0x4FFFFFFF)
#define PERIPHERALS_BASE_NS_END        (0xE0000000 - 1) //YX 20230202 porting to WE2.

/* Enable system reset request for CPU 0 */
#define ENABLE_CPU0_SYSTEM_RESET_REQUEST (1U << 8U)

/* To write into AIRCR register, 0x5FA value must be write to the VECTKEY field,
 * otherwise the processor ignores the write.
 */
#define SCB_AIRCR_WRITE_MASK ((0x5FAUL << SCB_AIRCR_VECTKEY_Pos))

/* Debug configuration flags */
#define SPNIDEN_SEL_STATUS (0x01u << 7)
#define SPNIDEN_STATUS     (0x01u << 6)
#define SPIDEN_SEL_STATUS  (0x01u << 5)
#define SPIDEN_STATUS      (0x01u << 4)
#define NIDEN_SEL_STATUS   (0x01u << 3)
#define NIDEN_STATUS       (0x01u << 2)
#define DBGEN_SEL_STATUS   (0x01u << 1)
#define DBGEN_STATUS       (0x01u << 0)

#define All_SEL_STATUS (SPNIDEN_SEL_STATUS | SPIDEN_SEL_STATUS | \
                        NIDEN_SEL_STATUS | DBGEN_SEL_STATUS)

enum tfm_plat_err_t enable_fault_handlers(void)
{
    /* Explicitly set secure fault priority to the highest */
    NVIC_SetPriority(SecureFault_IRQn, 0);

    /* Enables BUS, MEM, USG and Secure faults */
    SCB->SHCSR |= SCB_SHCSR_USGFAULTENA_Msk
                  | SCB_SHCSR_BUSFAULTENA_Msk
                  | SCB_SHCSR_MEMFAULTENA_Msk
                  | SCB_SHCSR_SECUREFAULTENA_Msk;
    return TFM_PLAT_ERR_SUCCESS;
}

enum tfm_plat_err_t system_reset_cfg(void)
{
    uint32_t reg_value = SCB->AIRCR;

    /* Enable system reset request for CPU 0, to be triggered via
     * NVIC_SystemReset function.
     */

    /* Clear SCB_AIRCR_VECTKEY value */
    reg_value &= ~(uint32_t)(SCB_AIRCR_VECTKEY_Msk);

#ifdef DEBUG //YX 20230804
    reg_value |= (uint32_t)(SCB_AIRCR_WRITE_MASK);
    reg_value &= (~SCB_AIRCR_SYSRESETREQS_Msk);
#else
    /* Enable system reset request only to the secure world */
    reg_value |= (uint32_t)(SCB_AIRCR_WRITE_MASK | SCB_AIRCR_SYSRESETREQS_Msk);
#endif

    SCB->AIRCR = reg_value;

    return TFM_PLAT_ERR_SUCCESS;
}

/*--------------------- NVIC interrupt NS/S configuration --------------------*/
#if 0 //YX 20230202 porting to WE2
enum tfm_plat_err_t nvic_interrupt_target_state_cfg(void)
{
    uint8_t i;

    /* Target every interrupt to NS; unimplemented interrupts will be WI */
    for (i = 0; i < (sizeof(NVIC->ITNS) / sizeof(NVIC->ITNS[0])); i++) {
        NVIC->ITNS[i] = 0xFFFFFFFF;
    }

    //MPC
    NVIC_ClearTargetState(AHB_SRAM2_MPC_IRQn);
    NVIC_ClearTargetState(OSPI_MPC_IRQn);
    NVIC_ClearTargetState(AXI_SRAM0_AXI_MPC_IRQn);
    NVIC_ClearTargetState(AXI_SRAM1_AXI_MPC_IRQn);
    NVIC_ClearTargetState(qspi_cache_Cache_MPC_IRQn);
    //PPC
	NVIC_ClearTargetState(APB_1_NS_PPC_IRQn);
	NVIC_ClearTargetState(APB_1_PS_PPC_IRQn);
	NVIC_ClearTargetState(AHB_3_PPC_IRQn);
	NVIC_ClearTargetState(APB0_NS_PPC_IRQn);
	NVIC_ClearTargetState(APB0_PS_PPC_IRQn);
	NVIC_ClearTargetState(AXI_ROM_AXI_PPC_IRQn);
	NVIC_ClearTargetState(APB_2_APB_PPC_IRQn);
	NVIC_ClearTargetState(qspi_cache_DWC_PPC_IRQn);
	NVIC_ClearTargetState(qspi_cache_ISP_PPC_IRQn);
	NVIC_ClearTargetState(AHB1M4_PPC_IRQn);
	NVIC_ClearTargetState(APB1_AON_PPC_IRQn);

	TZ_Set_ITS_byIRQNo(QSPI_spi_w_intp_IRQn);
	TZ_Set_ITS_byIRQNo(QSPI_ssi_intp_IRQn);

#ifdef IP_dma
#ifdef IP_INST_DMA1
	TZ_Set_ITS_byIRQNo(DMAC1_DMACINTERR_IRQn);
	TZ_Set_ITS_byIRQNo(DMAC1_DMACINTTC_IRQn);
	TZ_Set_ITS_byIRQNo(DMAC1_DMACINTR_IRQn);
#endif
#endif

#ifdef SECURE_UART1
    /* UART1 is a secure peripheral, so its IRQs have to target S state */
    NVIC_ClearTargetState(UART1_intr_IRQn);
    NVIC_ClearTargetState(UART1_intr_IRQn);
#endif

    return TFM_PLAT_ERR_SUCCESS;
}
#endif
/*----------------- NVIC interrupt enabling for S peripherals ----------------*/
#if 0 //YX 20230202 porting to WE2
enum tfm_plat_err_t nvic_interrupt_enable(void)
{
    int32_t ret = ARM_DRIVER_OK;
    int32_t i = 0;

    /* MPC interrupt enabling */
    mpc_clear_irq();
    ret = Driver_ISRAM0_MPC.EnableInterrupt();
    if (ret != ARM_DRIVER_OK) {
        ERROR_MSG("Failed to Enable MPC interrupt for ISRAM0!");
        return TFM_PLAT_ERR_SYSTEM_ERR;
    }

    ret = Driver_SRAM_MPC.EnableInterrupt();
    if (ret != ARM_DRIVER_OK) {
        ERROR_MSG("Failed to Enable MPC interrupt for SRAM!");
        return TFM_PLAT_ERR_SYSTEM_ERR;
    }

    NVIC_ClearPendingIRQ(MPC_IRQn);
    NVIC_EnableIRQ(MPC_IRQn);

    /* PPC interrupt enabling */
    ppc_clear_irq();

    for (i = 0; i < PPC_BANK_COUNT; i++)  {
        ret = ppc_bank_drivers[i]->EnableInterrupt();
        if (ret != ARM_DRIVER_OK) {
            ERROR_MSG("Failed to Enable interrupt on PPC");
            return TFM_PLAT_ERR_SYSTEM_ERR;
        }
    }

    NVIC_ClearPendingIRQ(PPC_IRQn);
    NVIC_EnableIRQ(PPC_IRQn);

#ifdef PSA_FF_TEST_SECURE_UART2
    NVIC_EnableIRQ(FF_TEST_UART_IRQ);
#endif
    return TFM_PLAT_ERR_SUCCESS;
}
#endif

#if 0 //YX 20230202 porting to WE2. 
enum tfm_plat_err_t init_debug(void)
{
    struct sse300_sysctrl_t *sysctrl =
                            (struct sse300_sysctrl_t *)SSE300_SYSCTRL_BASE_S;

#if defined(DAUTH_NONE)
    /* Set all the debug enable selector bits to 1 */
    sysctrl->secdbgset = All_SEL_STATUS;
    /* Set all the debug enable bits to 0 */
    sysctrl->secdbgclr =
                   DBGEN_STATUS | NIDEN_STATUS | SPIDEN_STATUS | SPNIDEN_STATUS;
#elif defined(DAUTH_NS_ONLY)
    /* Set all the debug enable selector bits to 1 */
    sysctrl->secdbgset = All_SEL_STATUS;
    /* Set the debug enable bits to 1 for NS, and 0 for S mode */
    sysctrl->secdbgset = DBGEN_STATUS | NIDEN_STATUS;
    sysctrl->secdbgclr = SPIDEN_STATUS | SPNIDEN_STATUS;
#elif defined(DAUTH_FULL)
    /* Set all the debug enable selector bits to 1 */
    sysctrl->secdbgset = All_SEL_STATUS;
    /* Set all the debug enable bits to 1 */
    sysctrl->secdbgset =
                   DBGEN_STATUS | NIDEN_STATUS | SPIDEN_STATUS | SPNIDEN_STATUS;
#else

#if !defined(DAUTH_CHIP_DEFAULT)
#error "No debug authentication setting is provided."
#endif

    /* Set all the debug enable selector bits to 0 */
    sysctrl->secdbgclr = All_SEL_STATUS;

    /* No need to set any enable bits because the value depends on
     * input signals.
     */
#endif
    return TFM_PLAT_ERR_SUCCESS;
}
#endif

/*------------------- SAU/IDAU configuration functions -----------------------*/
void sau_and_idau_cfg(void)
{
    /* Disables SAU */
    TZ_SAU_Disable();

    /* Configures SAU regions to be non-secure */
    SAU->RNR  = 0;
#if 0 //tfm org
    SAU->RBAR = (memory_regions.non_secure_partition_base & SAU_RBAR_BADDR_Msk);
    SAU->RLAR = (memory_regions.non_secure_partition_limit & SAU_RLAR_LADDR_Msk) | SAU_RLAR_ENABLE_Msk;
#else //we2 tz
    SAU->RBAR = (0x00000000 & SAU_RBAR_BADDR_Msk);
    SAU->RLAR = ((0x10000000 - 1) & SAU_RLAR_LADDR_Msk) | SAU_RLAR_ENABLE_Msk;
#endif

    SAU->RNR  = 1;
#if 0 //tfm org
    SAU->RBAR = (NS_DATA_START & SAU_RBAR_BADDR_Msk);
    SAU->RLAR = (NS_DATA_LIMIT & SAU_RLAR_LADDR_Msk) | SAU_RLAR_ENABLE_Msk;
#else //we2 tz
    SAU->RBAR = (0x20000000 & SAU_RBAR_BADDR_Msk);
    SAU->RLAR = ((0x30000000 - 1) & SAU_RLAR_LADDR_Msk) | SAU_RLAR_ENABLE_Msk;
#endif

    /* Configures veneers region to be non-secure callable */
    SAU->RNR  = 2;
    SAU->RBAR = (memory_regions.veneer_base & SAU_RBAR_BADDR_Msk);
    SAU->RLAR = (memory_regions.veneer_limit & SAU_RLAR_LADDR_Msk) | SAU_RLAR_ENABLE_Msk | SAU_RLAR_NSC_Msk;

    /* Configure the peripherals space */
    SAU->RNR  = 3;
    SAU->RBAR = (PERIPHERALS_BASE_NS_START & SAU_RBAR_BADDR_Msk);
    SAU->RLAR = (PERIPHERALS_BASE_NS_END & SAU_RLAR_LADDR_Msk) | SAU_RLAR_ENABLE_Msk;

    /* Enables SAU */
    TZ_SAU_Enable();

    //YX 20230202 porting to WE2. from SystemInit of system_ARMCM55.c
    hx_drv_scu_set_CM55M_IDAU(3, 0, 0, 1);
}

/*------------------- Memory configuration functions -------------------------*/
enum tfm_plat_err_t mpc_init_cfg(void)
{
#if defined(EMULATED_MPC_LAYOUT)
	uint32_t sram0_blks, sram1_blks, sram2_blks, flash1_blks, flash2_blks;
	uint32_t addr;
	uint8_t itcm_BLKSZ, dtcm_BLKSZ, itcm_blks, dtcm_blks,itcm_NUMBLKS, dtcm_NUMBLKS;
	TGU_PRESENT_E itcm_present, dtcm_present;

	EPII_Get_ITGU_CFG(&itcm_present, &itcm_NUMBLKS, &itcm_BLKSZ);
	itcm_blks = 1 << itcm_NUMBLKS;
	EPII_Get_DTGU_CFG(&dtcm_present, &dtcm_NUMBLKS, &dtcm_BLKSZ);
	dtcm_blks = 1 << dtcm_NUMBLKS;

	sram0_blks = SRAM0_SIZE/SRAM0_BLK_SIZE;
	sram1_blks = SRAM1_SIZE/SRAM1_BLK_SIZE;
	sram2_blks = SRAM2_SIZE/SRAM2_BLK_SIZE;
	flash1_blks = FLASH1_SIZE/FLASH1_BLK_SIZE;
	flash2_blks = FLASH2_SIZE/FLASH2_BLK_SIZE;

	/*ITCM*/  //S_CODE_START
	for(uint32_t blk_idx = 0; blk_idx < itcm_blks; blk_idx++)
	{
		//addr = BASE_ADDR_ITCM_ALIAS + blk_idx * ITGU_BLK_SIZE;
        EPII_Set_ITGU_LUT(blk_idx, TGU_LUT_SECURE);
	}
	/*DTCM*/  //S_DATA_START
	for(uint32_t blk_idx = 0; blk_idx < dtcm_blks; blk_idx++)
	{
		//addr = BASE_ADDR_DTCM_ALIAS + blk_idx * DTGU_BLK_SIZE;
        EPII_Set_DTGU_LUT(blk_idx, TGU_LUT_SECURE);
	}
	/*SRAM0*/ //NS_DATA_START
    for(uint32_t blk_idx = 0; blk_idx < sram0_blks; blk_idx++)
    {
        addr = BASE_ADDR_SRAM0 + blk_idx * SRAM0_BLK_SIZE;

        if((addr >= NS_DATA_START) && (addr <= NS_DATA_LIMIT))
            hx_drv_mpc_set_sram0_lut_byblkidx(blk_idx, MPC_LUT_STATUS_NONSECURE);
        else
            hx_drv_mpc_set_sram0_lut_byblkidx(blk_idx, MPC_LUT_STATUS_SECURE);
    }
	/*SRAM1*/ //NS_CODE_START
	for(uint32_t blk_idx = 0; blk_idx < sram1_blks; blk_idx++)
	{
        addr = BASE_ADDR_SRAM1 + blk_idx * SRAM1_BLK_SIZE;

        if((addr >= memory_regions.non_secure_partition_base) && (addr <= memory_regions.non_secure_partition_limit))
            hx_drv_mpc_set_sram1_lut_byblkidx(blk_idx, MPC_LUT_STATUS_NONSECURE);
        else
            hx_drv_mpc_set_sram1_lut_byblkidx(blk_idx, MPC_LUT_STATUS_SECURE);
	}
	/*SRAM2*/ //for emulated_flash_dev
    for(uint32_t blk_idx = 0; blk_idx < sram2_blks; blk_idx++)
    {
        //addr = BASE_ADDR_SRAM2 + blk_idx * SRAM2_BLK_SIZE;
        hx_drv_mpc_set_sram2_lut_byblkidx(blk_idx, MPC_LUT_STATUS_SECURE);
    }
	/*Flash1*/
    for(uint32_t blk_idx = 0; blk_idx < flash1_blks; blk_idx++)
    {
		//addr = BASE_ADDR_FLASH1_R_ALIAS + blk_idx * FLASH1_BLK_SIZE;
        hx_drv_mpc_set_flash1_lut_byblkidx(blk_idx, MPC_LUT_STATUS_SECURE);
    }
	/*Flash2*/
    for(uint32_t blk_idx = 0; blk_idx < flash2_blks; blk_idx++)
    {
		//addr = BASE_ADDR_FLASH2_ALIAS + blk_idx * FLASH2_BLK_SIZE;
        hx_drv_mpc_set_flash2_lut_byblkidx(blk_idx, MPC_LUT_STATUS_SECURE);
    }

    /* Add barriers to assure the MPC configuration is done before continue
     * the execution.
     */
    __DSB();
    __ISB();
#endif
    return TFM_PLAT_ERR_SUCCESS;
}

#if 0
void mpc_revert_non_secure_to_secure_cfg(void)
{
    /* Add barriers to assure the MPC configuration is done before continue
     * the execution.
     */
    __DSB();
    __ISB();
}
#endif

void mpc_clear_irq(void)
{
	hx_drv_mpc_set_sram0_irq_clear();
	hx_drv_mpc_set_sram1_irq_clear();
	hx_drv_mpc_set_sram2_irq_clear();
	hx_drv_mpc_set_flash1_irq_clear();
	hx_drv_mpc_set_flash2_irq_clear();
}

/*------------------- PPC configuration functions -------------------------*/
#if 0
extern void TZ_PPC_AHB1M4_CB(uint32_t event);
extern void TZ_PPC_AHB3_CB(uint32_t event);
extern void TZ_set_DMA1MSC(uint32_t state, uint32_t resp, uint32_t irq_enable);

enum tfm_plat_err_t ppc_init_cfg(void)
{
    PPC_APB0PS_CFG_T cfg;
    hx_drv_ppc_get_apb0ps(&cfg);
    cfg.uart0_ap = SCU_PPC_AP_ENABLE;
#ifdef IP_INST_NS_UART0
    cfg.uart0_state = SCU_PPC_NONSEC_NS;
#else
    cfg.uart0_state = SCU_PPC_NONSEC_SECURE;
#endif
    hx_drv_ppc_set_apb0ps(cfg);

    PPC_AHB1M4_CFG_T ahb1m4_cfg;
    hx_drv_ppc_get_ahb1m4(&ahb1m4_cfg);
    ahb1m4_cfg.dma1_state = 0;
    ahb1m4_cfg.dma1_ap = 1;
    ahb1m4_cfg.callback = NULL;
	ahb1m4_cfg.irq_enable = 1;
	ahb1m4_cfg.callback = &TZ_PPC_AHB1M4_CB;
    TZ_set_DMA1MSC(0, 0, 1);
    hx_drv_ppc_set_ahb1m4(ahb1m4_cfg);
    
    PPC_AHB3_CFG_T ahb3_cfg;
    hx_drv_ppc_get_ahb3(&ahb3_cfg);
    ahb3_cfg.sspi_host_state = 0;
    ahb3_cfg.sspi_host_ap = 1;
    ahb3_cfg.irq_enable = 1;
	ahb3_cfg.callback = &TZ_PPC_AHB3_CB;
    hx_drv_ppc_set_ahb3(ahb3_cfg);
    return TFM_PLAT_ERR_SUCCESS;
}
#endif

void hx_drv_ppc_apb1ns_state_configure(SCU_PPC_NONSEC_E state, uint32_t pos)
{
    PPC_APB1NS_CFG_T cfg;
    hx_drv_ppc_get_apb1ns(&cfg);
    
    if (pos & SYSTEM_TIMER0_PERIPH_PPC_APB1NS_POS_MASK)
        cfg.timer0_state = state;
    if (pos & SYSTEM_TIMER2_PERIPH_PPC_APB1NS_POS_MASK)
        cfg.timer2_state = state;
    if (pos & SYSTEM_TIMER4_PERIPH_PPC_APB1NS_POS_MASK)
        cfg.timer4_state = state;

    hx_drv_ppc_set_apb1ns(cfg);
}

void hx_drv_ppc_apb0ps_state_configure(SCU_PPC_NONSEC_E state, uint32_t pos)
{
    PPC_APB0PS_CFG_T cfg;
    hx_drv_ppc_get_apb0ps(&cfg);
    
    if (pos & SYSTEM_UART0_PERIPH_PPC_APB0PS_POS_MASK)
        cfg.uart0_state = state;

    hx_drv_ppc_set_apb0ps(cfg);
}

void hx_drv_ppc_apb1ns_secure_ap_configure(SCU_PPC_AP_E ap, uint32_t pos)
{
    PPC_APB1NS_CFG_T cfg;
    hx_drv_ppc_get_apb1ns(&cfg);
    
    if (pos & SYSTEM_TIMER0_PERIPH_PPC_APB1NS_POS_MASK) {
        cfg.timer0_ap = ap;
        cfg.timer0_state = SCU_PPC_NONSEC_SECURE;
	}
    if (pos & SYSTEM_TIMER2_PERIPH_PPC_APB1NS_POS_MASK) {
        cfg.timer2_ap = ap;
        cfg.timer2_state = SCU_PPC_NONSEC_SECURE;
	}
    if (pos & SYSTEM_TIMER4_PERIPH_PPC_APB1NS_POS_MASK) {
        cfg.timer4_ap = ap;
        cfg.timer4_state = SCU_PPC_NONSEC_SECURE;
	}

    hx_drv_ppc_set_apb1ns(cfg);
}

void hx_drv_ppc_apb0ps_secure_ap_configure(SCU_PPC_AP_E ap, uint32_t pos)
{
    PPC_APB0PS_CFG_T cfg;
    hx_drv_ppc_get_apb0ps(&cfg);
    
    if (pos & SYSTEM_UART0_PERIPH_PPC_APB0PS_POS_MASK) {
        cfg.uart0_ap = ap;
        cfg.uart0_state = SCU_PPC_NONSEC_SECURE;
	}

    hx_drv_ppc_set_apb0ps(cfg);
}

void ppc_configure_to_secure(enum ppc_bank_e bank, uint32_t pos)
{
	switch (bank)
	{
	case PPC_SP_APB1NS:
        hx_drv_ppc_apb1ns_state_configure(SCU_PPC_NONSEC_SECURE, pos);
		break;
	case PPC_SP_APB0PS:
        hx_drv_ppc_apb0ps_state_configure(SCU_PPC_NONSEC_SECURE, pos);
		break;
	default:
		break;
	}
}

void ppc_configure_to_non_secure(enum ppc_bank_e bank, uint32_t pos)
{
	switch (bank)
	{
	case PPC_SP_APB1NS:
        hx_drv_ppc_apb1ns_state_configure(SCU_PPC_NONSEC_NS, pos);
		break;
	case PPC_SP_APB0PS:
        hx_drv_ppc_apb0ps_state_configure(SCU_PPC_NONSEC_NS, pos);
		break;
	default:
		break;
	}
}

void ppc_en_secure_unpriv(enum ppc_bank_e bank, uint32_t pos)
{
	switch (bank)
	{
	case PPC_SP_APB1NS:
        hx_drv_ppc_apb1ns_secure_ap_configure(SCU_PPC_AP_ENABLE, pos); //unpriv and priv access
		break;
	case PPC_SP_APB0PS:
        hx_drv_ppc_apb0ps_secure_ap_configure(SCU_PPC_AP_ENABLE, pos); //unpriv and priv access
		break;
	default:
		break;
	}
}

void ppc_clr_secure_unpriv(enum ppc_bank_e bank, uint32_t pos)
{
	switch (bank)
	{
	case PPC_SP_APB1NS:
        hx_drv_ppc_apb1ns_secure_ap_configure(SCU_PPC_AP_DISABLE, pos); //secure and only priv access
		break;
	case PPC_SP_APB0PS:
        hx_drv_ppc_apb0ps_secure_ap_configure(SCU_PPC_AP_DISABLE, pos); //secure and only priv access
		break;
	default:
		break;
	}
}

void ppc_clear_irq(void)
{
	hx_drv_scu_set_dwc_ppc_irq_clear();
    hx_drv_scu_set_isp_ppc_irq_clear();
    hx_drv_scu_set_ahb3_ppc_irq_clear();
    hx_drv_scu_set_ahb1m4_ppc_irq_clear();
    hx_drv_scu_set_apb0ns_ppc_irq_clear();
    hx_drv_scu_set_apb0ps_ppc_irq_clear();
    hx_drv_scu_set_apb1ns_ppc_irq_clear();
    hx_drv_scu_set_apb1ps_ppc_irq_clear();
    hx_drv_scu_set_apb2_ppc_irq_clear();
    hx_drv_scu_set_rom_ppc_irq_clear();
    hx_drv_scu_set_apb1_aon_ppc_irq_clear();
}
