/*
 * Copyright (c) 2017-2022, Arm Limited. All rights reserved.
 * Copyright (c) 2022 Cypress Semiconductor Corporation (an Infineon company)
 * or an affiliate of Cypress Semiconductor Corporation. All rights reserved.
 *
 * SPDX-License-Identifier: BSD-3-Clause
 *
 */

#include "tfm_api.h"
#if (defined(TFM_NS_RTOS2_WRAPPER_RTOS2_RTX) || \
     defined(TFM_NS_RTOS2_WRAPPER_RTOS2_FREERTOS))
#include "cmsis_os2.h"
#endif
#include "cmsis_compiler.h"
#include "tfm_ns_interface.h"
#include "tfm_nsid_manager.h"
#include "test_app.h"
#include "tfm_plat_ns.h"
#include "driver/Driver_USART.h"
#include "device_cfg.h"
//#ifdef TFM_PARTITION_NS_AGENT_MAILBOX
//#include "tfm_multi_core_api.h"
//#include "tfm_ns_mailbox.h"
//#endif
#include "tfm_log.h"
#include "uart_stdout.h"
#if (CONFIG_TFM_FLOAT_ABI >= 1)
#include "cmsis.h"
#endif

#include "WE2_core.h"
#include "board.h"
#include "veneer_sys_ctrl.h"
#include "hx_drv_scu_export.h"
#if defined(TFM_NS_APP_PMU_DPD_SAMPLE)
#include "pmu_dpd_sample.h"
#endif

#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL) || defined(TFM_NS_RTOS2_WRAPPER_RTOS2_FREERTOS)
#if defined(TFM_NS_MANAGE_NSID)
#error "TFM_NS_MANAGE_NSID is defined"
#endif
#endif

#if defined(TFM_PARTITION_NS_AGENT_MAILBOX)
#error "TFM_PARTITION_NS_AGENT_MAILBOX is defined"
#endif

#if defined(TFM_MULTI_CORE_NS_OS_MAILBOX_THREAD)
#error "TFM_MULTI_CORE_NS_OS_MAILBOX_THREAD is defined"
#endif

__attribute__((noreturn))
static void hello_app(void *argument)
{
    UNUSED_VARIABLE(argument);

    LOG_MSG("TFM NS Hello World!!\r\n");
    uint32_t version1;
    uint32_t version2;
    uint32_t status;

    LOG_MSG("Get Hardware Version from Secure Partition HX Service\r\n");
    status = veneer_sys_get_version(&version1, &version2);

    if (status == SCU_NO_ERROR)
        LOG_MSG("Success\r\n");
    else
        LOG_MSG("Failure 0x%X\r\n", status);

    LOG_MSG("Version 0x%X 0x%X\r\n", version1, version2);
#if defined(TFM_NS_APP_PMU_DPD_SAMPLE)
    pmu_dpd_app();
#endif
    for (;;) {
    }
}

/**
 * \brief Modified table template for user defined SVC functions
 *
 * \details RTX has a weak definition of osRtxUserSVC, which
 *          is overridden here
 */
#if defined(__ARMCC_VERSION)
#if (__ARMCC_VERSION == 6110004)
/* Workaround needed for a bug in Armclang 6.11, more details at:
 * http://www.keil.com/support/docs/4089.htm
 */
__attribute__((section(".gnu.linkonce")))
#endif

/* Avoids the semihosting issue */
#if (__ARMCC_VERSION >= 6010050)
__asm("  .global __ARM_use_no_argv\n");
#endif
#endif

#if (defined(TFM_NS_RTOS2_WRAPPER_RTOS2_RTX) || \
     defined(TFM_NS_RTOS2_WRAPPER_RTOS2_FREERTOS))
/**
 * \brief List of RTOS thread attributes
 */
static const osThreadAttr_t thread_attr = {
    .name = "test_thread",
    .stack_size = 4096U,
    .tz_module = ((TZ_ModuleId_t)TFM_DEFAULT_NSID)
};
/**
 * \brief Static globals to hold RTOS related quantities,
 *        main thread
 */

#if (defined(TEST_FRAMEWORK_NS) || \
     defined(PSA_API_TEST_NS) || \
     defined(TEST_NS_FLIH_HX))
static osThreadFunc_t thread_func = test_app;
#else
static osThreadFunc_t thread_func = hello_app;
#endif
#endif

#ifdef CONFIG_TFM_USE_TRUSTZONE
extern uint32_t tfm_ns_interface_init(void);
#endif

/**
 * \brief Platform peripherals and devices initialization.
 *        Can be overridden for platform specific initialization.
 *
 * \return  ARM_DRIVER_OK if the initialization succeeds
 */

#if defined(TEST_NS_FPU)
#include "cmsis.h"
#include "tfm_peripherals_def.h"
#include "uart_stdout.h"
//#include "Driver_USART.h"
#include "test_interrupt.h"
__WEAK int32_t tfm_ns_platform_init (void)
{
    /* Register FPU non-secure test interrupt handler */
#if 0 //YX 20230112 Removed. startup_an552.c vector table has registered
    NVIC_SetVector(TFM_FPU_NS_TEST_IRQ, (uint32_t)TFM_FPU_NS_TEST_Handler);
#endif

    /* Enable FPU non-secure test interrupt */
    NVIC_EnableIRQ(TFM_FPU_NS_TEST_IRQ);

#if defined(TFM_PARTITION_HX)
    board_init();
    stdio_init();
#endif
    return ARM_DRIVER_OK;
}
#else
__WEAK int32_t tfm_ns_platform_init(void)
{
#if defined(TFM_PARTITION_HX)
    board_init();
    stdio_init();
#endif
    return ARM_DRIVER_OK;
}
#endif

/**
 * \brief Platform peripherals and devices de-initialization.
 *        Can be overridden for platform specific initialization.
 *
 * \return  ARM_DRIVER_OK if the de-initialization succeeds
 */
__WEAK int32_t tfm_ns_platform_uninit(void)
{
    stdio_uninit();

    return ARM_DRIVER_OK;
}

__WEAK int32_t tfm_ns_cp_init(void)
{
#if (CONFIG_TFM_FLOAT_ABI >= 1)
#ifdef __GNUC__
    /* Enable NSPE privileged and unprivilged access to the FP Extension */
    SCB->CPACR |= (3U << 10U*2U)     /* enable CP10 full access */
                  | (3U << 11U*2U);  /* enable CP11 full access */
#endif
#endif
    return ARM_DRIVER_OK;
}

#if defined(TFM_NS_MANAGE_NSID)
#error "Not Support TFM_NS_MANAGE_NSID"
//Calling service must execute in thread for TFM_NS_MANAGE_NSID
#endif

/**
 * \brief tfm_ns_app() function
 */
__WEAK void tfm_ns_app(void)
{
    if (tfm_ns_platform_init() != ARM_DRIVER_OK) {
        /* Avoid undefined behavior if platform init failed */
        while(1);
    }

    if (tfm_ns_cp_init() != ARM_DRIVER_OK) {
        /* Avoid undefined behavior if co-porcessor init failed */
        while(1);
    }

#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
    LOG_MSG("\r\n\033[1;34m[Bare Metal] Non-Secure system starting...\033[0m\r\n");

#if defined(__GNU__)
    LOG_MSG("Compiler Version: ARM GNU, %s\r\n", __VERSION__);
#else
    LOG_MSG("Compiler Version: ARM CLANG, %s\r\n", __VERSION__);
#endif

#if (defined(TEST_FRAMEWORK_NS) || \
     defined(PSA_API_TEST_NS) || \
     defined(TEST_NS_FLIH_HX))
    test_app(NULL);
    /* Reached only in case of error */
    for (;;) {
    }
#else
    hello_app(NULL);
    /* Reached only in case of error */
    for (;;) {
    }
#endif

#else //#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
#if defined(TFM_NS_RTOS2_WRAPPER_RTOS2_FREERTOS)
    LOG_MSG("\033[1;34m[RTOS2 FreeRTOS] Non-Secure system starting...\033[0m\r\n");
#else
    LOG_MSG("\033[1;34m[RTOS2 RTX] Non-Secure system starting...\033[0m\r\n");
#endif

#if defined(__GNU__)
    LOG_MSG("Compiler Version: ARM GNU, %s\r\n", __VERSION__);
#else
    LOG_MSG("Compiler Version: ARM CLANG, %s\r\n", __VERSION__);
#endif

    (void) osKernelInitialize();

#ifdef CONFIG_TFM_USE_TRUSTZONE
    /* Initialize the TFM NS interface */
    tfm_ns_interface_init();
#endif

    (void) osThreadNew(thread_func, NULL, &thread_attr);

    osKernelState_t state = osKernelGetState();

    if (state == osKernelReady) {
        (void) osKernelStart();
    }

    /* Reached only in case of error */
    for (;;) {
    }
#endif //#if defined(TFM_NS_RTOS2_WRAPPER_BARE_METAL)
}
